@@ -39,9 +39,6 @@ class Incrementer:
 
             self.progress_pipe.send((self.finished, self.total, section_name))
 
-            # TODO: remove this!
-            self.logger.debug("%s", (self.finished, self.total, section_name))
-
     def set_values(self, finished, total):
         if self.finished is not None:
             if finished != self.finished:
@@ -56,170 +53,6 @@ class Incrementer:
         self.total = total
 
 
-def get_device_info(device, allow_reconnect, diskcache=None,
-                    progress_pipe=None):
-    device_logger = hyperborea.proxy.get_device_logger(logger, device)
-    incrementer = Incrementer(progress_pipe, device_logger)
-
-    try:
-        # get all of the parameters needed for cache interaction
-        serial_number = device.get_serial_number()
-        protocol_type = device.device.protocol_type
-        build_info = device.get_build_info()
-        build_date = device.get_build_date()
-        nvm_hash = try_optional(device.get_nvm_hash)
-        setting_hash = try_optional(device.get_setting_hash)
-
-        finished_commands = 4
-        total_commands = 4
-
-        # load setting_info from cache, if applicable
-        setting_key = (serial_number, protocol_type, build_info, build_date,
-                       setting_hash)
-        if setting_hash is None and not allow_reconnect:
-            # not a simple reconnect, and the device is old-style
-            setting_info = {}
-        elif diskcache is not None:
-            setting_info = diskcache.get(setting_key, default={})
-            device_logger.debug("got setting_info from cache: %s",
-                                setting_info)
-        else:
-            setting_info = {}
-
-        # load nvm_info from cache, if applicable
-        nvm_key = (serial_number, nvm_hash)
-        if nvm_hash is None:
-            nvm_info = {}
-        elif diskcache is not None:
-            nvm_info = diskcache.get(nvm_key, default={})
-            device_logger.debug("got nvm_info from cache: %s", nvm_info)
-        else:
-            nvm_info = {}
-
-        # these values are free, or have already been fetched, and shouldn't be
-        # mixed into setting_info (or they would be saved to the cache).
-        state_info = {
-            'serial_number': serial_number,
-            'build_info': build_info,
-            'build_date': build_date,
-            'nvm_hash': nvm_hash,
-            'setting_hash': setting_hash,
-            'location_string': device.get_location_string(),
-            'library_protocol_version': asphodel.protocol_version_string,
-            'library_build_info': asphodel.build_info,
-            'library_build_date': asphodel.build_date,
-            'supports_rf_power': device.supports_rf_power_commands(),
-            'supports_radio': device.supports_radio_commands(),
-            'supports_remote': device.supports_remote_commands(),
-            'supports_bootloader': device.supports_bootloader_commands(),
-            'max_incoming_param_length':
-                device.get_max_incoming_param_length(),
-            'max_outgoing_param_length':
-                device.get_max_outgoing_param_length(),
-            'stream_packet_length': device.get_stream_packet_length(),
-        }
-
-        nvm_getters = [
-            get_nvm
-        ]
-
-        setting_getters = [
-            single_call('protocol_version', "get_protocol_version_string"),
-            single_call('board_info', "get_board_info"),
-            single_call('chip_family', "get_chip_family"),
-            single_call('chip_model', "get_chip_model"),
-            single_call('chip_id', "get_chip_id"),
-            single_call('bootloader_info', "get_bootloader_info"),
-            single_call('commit_id', "get_commit_id", optional=True),
-            single_call('repo_branch', "get_repo_branch", optional=True),
-            single_call('repo_name', "get_repo_name", optional=True),
-            single_call('nvm_modified', "get_nvm_modified", optional=True),
-            get_custom_enums,
-            get_setting_categories,
-            get_streams,
-            get_channels,
-            get_channel_calibrations,
-            get_supplies,
-            get_ctrl_vars,
-            get_settings,
-            get_rf_power_ctrl_vars,
-            get_radio_ctrl_vars,
-            get_radio_scan_power,
-            get_radio_default_serial,
-            get_led_settings,
-            get_rgb_settings,
-            get_supply_results,
-            get_ctrl_var_settings,
-            get_rf_power_status,
-            get_device_mode,
-        ]
-
-        # make a copy of everything
-        merged_dict = state_info.copy()
-        merged_dict.update(nvm_info)
-        merged_dict.update(setting_info)
-
-        info_and_getters = [(nvm_info, nvm_getters),
-                            (setting_info, setting_getters)]
-
-        generators_to_process = []
-
-        try:
-            for info_dict, getters in info_and_getters:
-                for get_fn in getters:
-                    gen = get_fn(device, merged_dict, incrementer)
-
-                    # do the first call on the generator
-                    try:
-                        finished, remaining, d = next(gen)
-                    except StopIteration:
-                        continue
-
-                    finished_commands += finished
-                    total_commands += finished + remaining
-
-                    # push the updates into the dicts
-                    merged_dict.update(d)
-                    info_dict.update(d)
-
-                    generators_to_process.append((gen, info_dict, remaining))
-
-            incrementer.set_values(finished_commands, total_commands)
-
-            for gen, info_dict, remaining in generators_to_process:
-                # do this now in case we get a StopIteration
-                finished_commands += remaining
-
-                try:
-                    d = next(gen)
-                except StopIteration:
-                    continue
-
-                # push the updates into the dicts
-                merged_dict.update(d)
-                info_dict.update(d)
-
-                incrementer.set_values(finished_commands, total_commands)
-
-        finally:
-            # store back to cache
-            if diskcache is not None:
-                device_logger.debug("storing setting_info to %s", setting_key)
-                diskcache.set(setting_key, setting_info)
-                if nvm_hash is not None:
-                    device_logger.debug("storing nvm_info to %s", nvm_key)
-                    diskcache.set(nvm_key, nvm_info)
-
-    except Exception:
-        device_logger.exception("Unhandled exception in get_device_info()")
-        return {}
-    finally:
-        if progress_pipe:
-            progress_pipe.close()
-
-    return merged_dict
-
-
 def single_call(key_name, func_name, optional=False):
     def func(device, device_info, incrementer):
         if key_name in device_info:
@@ -439,6 +272,35 @@ def get_nvm(device, device_info, incrementer):
            'user_tag_1': user_tag_1, 'user_tag_2': user_tag_2}
 
 
+def get_nvm_active_scan(device, device_info, incrementer):
+    if device_info['nvm_hash']:
+        # it's worth fetching the whole nvm because it'll be saved
+        yield from get_nvm(device, device_info, incrementer)
+        return
+
+    yield 0, 3, {}
+
+    tag_locations = device.get_user_tag_locations()
+    incrementer.increment(1, 'tag_locations')
+
+    try:
+        t1 = device.read_user_tag_string(*tag_locations[0])
+    except UnicodeDecodeError:
+        t1 = None
+
+    incrementer.increment(1, 'user_tag_1')
+
+    try:
+        t2 = device.read_user_tag_string(*tag_locations[1])
+    except UnicodeDecodeError:
+        t2 = None
+
+    incrementer.increment(1, 'user_tag_2')
+
+    yield {'tag_locations': tag_locations,
+           'user_tag_1': t1, 'user_tag_2': t2}
+
+
 def get_led_settings(device, device_info, incrementer):
     yield from array_call('led_settings', device_info, incrementer,
                           device.get_led_count,
@@ -525,3 +387,188 @@ def get_device_mode(device, device_info, incrementer):
             raise
     incrementer.increment(1, 'device_mode')
     yield d
+
+
+def get_device_info_actual(device, device_logger, allow_reconnect, diskcache,
+                           progress_pipe, setting_getters, nvm_getters):
+    incrementer = Incrementer(progress_pipe, device_logger)
+
+    try:
+        serial_number = device.get_serial_number()
+        if not serial_number:
+            device_logger.error("No serial number when fetching device info")
+            return {}
+
+        # get all of the parameters needed for cache interaction
+        protocol_type = device.device.protocol_type
+        build_info = device.get_build_info()
+        build_date = device.get_build_date()
+        nvm_hash = try_optional(device.get_nvm_hash)
+        setting_hash = try_optional(device.get_setting_hash)
+
+        finished_commands = 4
+        total_commands = 4
+
+        # load setting_info from cache, if applicable
+        setting_key = (serial_number, protocol_type, build_info, build_date,
+                       setting_hash)
+        if setting_hash is None and not allow_reconnect:
+            # not a simple reconnect, and the device is old-style
+            setting_info = {}
+        elif diskcache is not None:
+            setting_info = diskcache.get(setting_key, default={})
+            device_logger.debug("got setting_info from cache: %s",
+                                setting_info)
+        else:
+            setting_info = {}
+
+        # load nvm_info from cache, if applicable
+        nvm_key = (serial_number, nvm_hash)
+        if nvm_hash is None:
+            nvm_info = {}
+        elif diskcache is not None:
+            nvm_info = diskcache.get(nvm_key, default={})
+            device_logger.debug("got nvm_info from cache: %s", nvm_info)
+        else:
+            nvm_info = {}
+
+        # these values are free, or have already been fetched, and shouldn't be
+        # mixed into setting_info (or they would be saved to the cache).
+        state_info = {
+            'serial_number': serial_number,
+            'build_info': build_info,
+            'build_date': build_date,
+            'nvm_hash': nvm_hash,
+            'setting_hash': setting_hash,
+            'location_string': device.get_location_string(),
+            'library_protocol_version': asphodel.protocol_version_string,
+            'library_build_info': asphodel.build_info,
+            'library_build_date': asphodel.build_date,
+            'supports_rf_power': device.supports_rf_power_commands(),
+            'supports_radio': device.supports_radio_commands(),
+            'supports_remote': device.supports_remote_commands(),
+            'supports_bootloader': device.supports_bootloader_commands(),
+            'max_incoming_param_length':
+                device.get_max_incoming_param_length(),
+            'max_outgoing_param_length':
+                device.get_max_outgoing_param_length(),
+            'stream_packet_length': device.get_stream_packet_length(),
+        }
+
+        # make a copy of everything
+        merged_dict = state_info.copy()
+        merged_dict.update(nvm_info)
+        merged_dict.update(setting_info)
+
+        info_and_getters = [(nvm_info, nvm_getters),
+                            (setting_info, setting_getters)]
+
+        generators_to_process = []
+
+        try:
+            for info_dict, getters in info_and_getters:
+                for get_fn in getters:
+                    gen = get_fn(device, merged_dict, incrementer)
+
+                    # do the first call on the generator
+                    try:
+                        finished, remaining, d = next(gen)
+                    except StopIteration:
+                        continue
+
+                    finished_commands += finished
+                    total_commands += finished + remaining
+
+                    # push the updates into the dicts
+                    merged_dict.update(d)
+                    info_dict.update(d)
+
+                    generators_to_process.append((gen, info_dict, remaining))
+
+            incrementer.set_values(finished_commands, total_commands)
+
+            for gen, info_dict, remaining in generators_to_process:
+                # do this now in case we get a StopIteration
+                finished_commands += remaining
+
+                try:
+                    d = next(gen)
+                except StopIteration:
+                    continue
+
+                # push the updates into the dicts
+                merged_dict.update(d)
+                info_dict.update(d)
+
+                incrementer.set_values(finished_commands, total_commands)
+
+        finally:
+            # store back to cache
+            if diskcache is not None:
+                diskcache.set(setting_key, setting_info)
+                if nvm_hash is not None:
+                    diskcache.set(nvm_key, nvm_info)
+
+    except Exception:
+        device_logger.exception("Unhandled exception fetching device info")
+        return {}
+    finally:
+        if progress_pipe:
+            progress_pipe.close()
+
+    device_logger.info("Fetched device information using %s commands",
+                       finished_commands)
+
+    return merged_dict
+
+
+default_setting_getters = [
+    single_call('protocol_version', "get_protocol_version_string"),
+    single_call('board_info', "get_board_info"),
+    single_call('chip_family', "get_chip_family"),
+    single_call('chip_model', "get_chip_model"),
+    single_call('chip_id', "get_chip_id"),
+    single_call('bootloader_info', "get_bootloader_info"),
+    single_call('commit_id', "get_commit_id", optional=True),
+    single_call('repo_branch', "get_repo_branch", optional=True),
+    single_call('repo_name', "get_repo_name", optional=True),
+    single_call('nvm_modified', "get_nvm_modified", optional=True),
+    get_custom_enums,
+    get_setting_categories,
+    get_streams,
+    get_channels,
+    get_channel_calibrations,
+    get_supplies,
+    get_ctrl_vars,
+    get_settings,
+    get_rf_power_ctrl_vars,
+    get_radio_ctrl_vars,
+    get_radio_scan_power,
+    get_radio_default_serial,
+    get_led_settings,
+    get_rgb_settings,
+    get_supply_results,
+    get_ctrl_var_settings,
+    get_rf_power_status,
+    get_device_mode,
+]
+
+active_scan_setting_getters = [
+    single_call('board_info', "get_board_info"),
+    single_call('bootloader_info', "get_bootloader_info"),
+]
+
+
+def get_active_scan_info(device, remote, diskcache=None):
+    device_logger = hyperborea.proxy.get_device_logger(logger, device)
+    return get_device_info_actual(
+        remote, device_logger, False, diskcache, None,
+        active_scan_setting_getters, [get_nvm_active_scan])
+
+
+def get_device_info(device, allow_reconnect, diskcache=None,
+                    progress_pipe=None):
+    device_logger = hyperborea.proxy.get_device_logger(logger, device)
+    return get_device_info_actual(
+        device, device_logger, allow_reconnect, diskcache, progress_pipe,
+        default_setting_getters, [get_nvm])
